#include <Wire.h>
#include <max6675.h>
#include <EEPROM.h>
#include <PID_v1.h>
#include <UTFT.h>

UTFT myGLCD(CTE32HR, 38, 39, 40, 41);

extern uint8_t SmallFont[];
extern uint8_t BigFont[];
extern uint8_t BigFontRus[]; //Кирилица
extern uint8_t SevenSegNumFont[];

int er1=1;
int er2=1;
int reg1;
int reg2;
boolean out1;
boolean out2;
int pwr_TOP;     //максимальная мощность верхнего нагревателя
int pwr_BOTTOM;  //максимальная мощность нижнего нагревателя

//RelayPin "1"-ВЕРХНИЙ нагреватель
//RelayPin "2"-НИЖНИЙ нагреватель
#define RelayPin1 6  //назначаем пин "ВЕРХНЕГО" нагревателя
#define RelayPin2 7  //назначаем пин "НИЖНЕГО" нагревателя

// Выходы реле
#define P1_PIN 9  //назначаем пин реле 1
#define P2_PIN 10  //назначаем пин реле 2
#define P3_PIN 11  //назначаем пин реле 3
#define P4_PIN 12  //назначаем пин реле 4
int tableSize;

//current editing step pointer
int editStep = 0;

int buzzerPin = 8;

//Назначаем пины кнопок управления
int upSwitchPin = 18;
int downSwitchPin = 19;
int cancelSwitchPin = 20;
int okSwitchPin = 21;

//состояние кнопок по умолчанию
int upSwitchState = 0;
int downSwitchState = 0;
int cancelSwitchState = 0;
int okSwitchState = 0;

//переменные для кнопок
long ms_button = 0;
boolean  button_state = false;
boolean  button_long_state = false;

//profile stuff
byte currentProfile = 1;
int currentStep = 1;
byte profileSteps;

double rampRateStep[9];

int dwellTimerStep[9];

int kp1;
int ki1;
int kd1;
int kp2;
int ki2;
int kd2;
double startTemp;

int setpointRamp;
boolean TopStart = false;
int flag = 0;  //флаг для фиксации стартовой температуры
int x=1;  //переменная для перехода на нужный шаг при горячей плате
int temperatureStep[9];
int eepromAddress = 0;//начало eepromaddress

long previousMillis; //это для счетчиков

double counter;

//these are the different states of the sketch. We call different ones depending on conditions
// ***** TYPE DEFINITIONS *****
typedef enum REFLOW_STATE
{
  REFLOW_STATE_IDLE,
  REFLOW_STATE_MENU_STEPS,
  REFLOW_STATE_MENU_TABLE_SIZE,
  REFLOW_STATE_MENU_BOTTOM_HEAT,
  REFLOW_STATE_MENU_BOTTOM_PWR,
  REFLOW_STATE_MENU_TOP_PWR,
  REFLOW_STATE_MENU_STEP_RAMP,
  REFLOW_STATE_MENU_STEP_TARGET,
  REFLOW_STATE_MENU_STEP_DWELL,
  REFLOW_STATE_MENU_BOTTOM_P,
  REFLOW_STATE_MENU_BOTTOM_I,
  REFLOW_STATE_MENU_BOTTOM_D,
  REFLOW_STATE_MENU_TOP_P,
  REFLOW_STATE_MENU_TOP_I,
  REFLOW_STATE_MENU_TOP_D,
  REFLOW_STATE_STEP_RAMP,
  REFLOW_STATE_TABLE_SIZE,
  REFLOW_STATE_STEP,
  REFLOW_STATE_STEP_DWELL,
  REFLOW_STATE_COMPLETE,
  REFLOW_STATE_ERROR
}
reflowState_t;

typedef enum REFLOW_STATUS //this is simply to check if reflow should be running or not
{
  REFLOW_STATUS_OFF,
  REFLOW_STATUS_ON
}
reflowStatus_t;

#define SENSOR_SAMPLING_TIME 1000 //read tc every second
reflowStatus_t reflowStatus;
// Reflow oven controller state machine state variable
reflowState_t reflowState;

//TC read timer variables
//unsigned long nextCheck1;
unsigned long nextRead1; //переменная для обновления текущей температуры
//PID stuff

double Setpoint1, Input1, Output1;
//Specify the links and initial tuning parameters
PID myPID1(&Input1, &Output1, &Setpoint1, kp1, ki1, kd1, DIRECT);
int WindowSize = 100;
unsigned long windowStartTime;
//PID stuff
double Setpoint2, Input2, Output2;
//Specify the links and initial tuning parameters
PID myPID2(&Input2, &Output2, &Setpoint2, kp2, ki2, kd2, DIRECT);

//Alarm state boolean
boolean alarmOn = false;

//Update whole screen boolean
boolean updateScreen = true;

//назначаем пины уселителя термопары MAX6675 "ВЕРХНЕГО" нагревателя   clk=sck cs=cs do=so
int thermoCLK = 48;  //=sck
int thermoCS = 47;   //=cs
int thermoDO = 46;   //=so
//назначаем пины уселителя термопары MAX6675 "НИЖНЕГО" нагревателя clk=sck cs=cs do=so
int thermoCLK2 = 44;  //=sck
int thermoCS2 = 43;   //=cs
int thermoDO2 = 42;   //=so

// переменные для калмана "ВЕРХНЕГО" нагревателя
float varTerm1 = 0.25;  // среднее отклонение (ищем в excel)
float varProcess1 = 0.0125; // скорость реакции на изменение (подбирается вручную)
float Pc1 = 0.0;
float G1 = 0.0;
float P1 = 1.0;
float Xp1 = 0.0;
float Zp1 = 0.0;
float Xe1 = 0.0;

// переменные для калмана "НИЖНЕГО" нагревателя
float varTerm2 = 0.25;  // среднее отклонение (ищем в excel)
float varProcess2 = 0.0125; // скорость реакции на изменение (подбирается вручную)
float Pc2 = 0.0;
float G2 = 0.0;
float P2 = 1.0;
float Xp2 = 0.0;
float Zp2 = 0.0;
float Xe2 = 0.0;
// переменные для калмана
int tc1;
int tc2;

MAX6675 thermocouple1(thermoCLK, thermoCS, thermoDO);//термопара "ВЕРХНЕГО" нагревателя
MAX6675 thermocouple2(thermoCLK2, thermoCS2, thermoDO2);//термопара "НИЖНЕГО" нагревателя

void loadProfile()//this function loads whichever profile currentProfile variable is set to
{
  profileSteps = EEPROM.read((currentProfile - 1) * 29);
  Setpoint2 = EEPROM.read((currentProfile - 1) * 29 + 1);
  for (int i = 0; i < 9; i + 1) {
    rampRateStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 2);
    i++;
  }
  for (int i = 0; i < 9; i + 1) {
    dwellTimerStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 11) * 5;
    i++;
  }
  for (int i = 0; i < 9; i + 1) {
    temperatureStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 20);
    i++;
  }
  kp1 = EEPROM.read((currentProfile - 1) * 6 + 122);
  ki1 = EEPROM.read((currentProfile - 1) * 6 + 123);
  kd1 = EEPROM.read((currentProfile - 1) * 6 + 124);
  kp2 = EEPROM.read((currentProfile - 1) * 6 + 125);
  ki2 = EEPROM.read((currentProfile - 1) * 6 + 126);
  kd2 = EEPROM.read((currentProfile - 1) * 6 + 127);
  tableSize = EEPROM.read((currentProfile - 1) * 6 + 128);
  pwr_TOP = EEPROM.read((currentProfile - 1) * 6 + 129);
  pwr_BOTTOM = EEPROM.read((currentProfile - 1) * 6 + 130);

  return;
}


void setup()
{
  Serial.begin(9600);
  
//setup pins as input for buttons
  pinMode (upSwitchPin, INPUT);
  digitalWrite(upSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (downSwitchPin, INPUT);
  digitalWrite(downSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (cancelSwitchPin, INPUT);
  digitalWrite(cancelSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (okSwitchPin, INPUT);
  digitalWrite(okSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (buzzerPin, OUTPUT);
//setup reley pins as outputs
  pinMode(P1_PIN, OUTPUT);
  digitalWrite(P1_PIN, HIGH); //подключаем подтягивающий резистор
  pinMode(P2_PIN, OUTPUT);
  digitalWrite(P2_PIN, HIGH); //подключаем подтягивающий резистор
  pinMode(P3_PIN, OUTPUT);
  digitalWrite(P3_PIN, HIGH); //подключаем подтягивающий резистор
  pinMode(P4_PIN, OUTPUT);
  digitalWrite(P4_PIN, HIGH); //подключаем подтягивающий резистор
  
  myGLCD.InitLCD();
  myGLCD.clrScr();
  myGLCD.setFont(BigFont);
  myGLCD.setColor(VGA_GREEN);
  myGLCD.print("ARDUINO MEGA2560",CENTER, 100);
  myGLCD.setColor(VGA_RED);
  myGLCD.print("REWORK STATION",CENTER, 130);
  myGLCD.setColor(250, 180, 000);
  myGLCD.print("TFT-LCD 480*320 MAX6675",CENTER, 160);
  //Мелодия приветствия
  tone(buzzerPin, 523);
  delay(200);
  tone(buzzerPin, 659);
  delay(200);
  tone(buzzerPin, 784);
  delay(200);
  tone(buzzerPin, 1046);
  delay(200);
  noTone(buzzerPin);
  // wait for MAX chips to stabilize and splash screen
  delay(1000);
  myGLCD.clrScr();
  
//setup ssr pins as outputs
  pinMode(RelayPin1, OUTPUT);
  pinMode(RelayPin2, OUTPUT);  

  //windowStartTime = millis()
  nextRead1 = millis();

  myPID1.SetOutputLimits(0, WindowSize);//myPID1 = top heater PID loop
  myPID2.SetOutputLimits(0, WindowSize);
  myPID1.SetMode(AUTOMATIC);
  myPID2.SetMode(AUTOMATIC);
  }

int i = 0;

void loop()
{
  //Считываем состояние кнопок управления
  upSwitchState = digitalRead(upSwitchPin);
  downSwitchState = digitalRead(downSwitchPin);
  cancelSwitchState = digitalRead(cancelSwitchPin);
  okSwitchState = digitalRead(okSwitchPin);
  //----------------------------------------------------------------------------------
  if (upSwitchState == HIGH) {
    upSwitchState = LOW;
  } else {
    upSwitchState = HIGH;
   
  }
  if (downSwitchState == HIGH) {
    downSwitchState = LOW;
  } else {
    downSwitchState = HIGH;
  }
  if (cancelSwitchState == HIGH) {
    cancelSwitchState = LOW;
  } else {
    cancelSwitchState = HIGH;
  }
  if (okSwitchState == HIGH) {
    okSwitchState = LOW;
  } else {
    okSwitchState = HIGH; 
  }
//-------------------------------------------------------------------------------

  //
  unsigned long currentMillis = millis();
      
  int SP1 = Setpoint1;
  int SP2 = Setpoint2;

//отключил бузер при нажатии на кнопки (нервирует), кому надо раскоментируйте ниже 6 строк
  //if (upSwitchState == HIGH || downSwitchState == HIGH || cancelSwitchState == HIGH || okSwitchState == HIGH)
  //{
     //tone(buzzerPin, 1045);
     //delay(100);
     //noTone(buzzerPin);
  //}
  if (reflowState == REFLOW_STATE_COMPLETE || alarmOn) {
    if (i < 2 && cancelSwitchState == LOW) {
      alarmOn = true;
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      i++;
    }
    else {
      i = 0;
      alarmOn = false;
    }
  }

  switch (reflowState)
  {
    case REFLOW_STATE_IDLE:
     currentStep = 1;
     counter = 0;
     setpointRamp = 0;
     x = 1;             //устанавливаем переменную в исходное состояние
     flag = 0;         //после отановки профиля сбрасываем флаг
     Output1=0;
     Output2=0;

      if (millis() > nextRead1)
      {
        // Read thermocouples next sampling period
        nextRead1 += SENSOR_SAMPLING_TIME;
        Input1 = filter1(thermocouple1.readCelsius());
        Input2 = filter2(thermocouple2.readCelsius());
        tc1 = Input1;
        tc2 = Input2;
        if (isnan(Input1)) {
          myGLCD.setFont(BigFont);
          myGLCD.setColor(VGA_RED);
          myGLCD.print("ERORR",360, 140);
        } else {
          myGLCD.setFont(SevenSegNumFont);
          myGLCD.setColor(VGA_SILVER);
          myGLCD.printNumI(tc1,350, 120);
        }
        if (isnan(Input2)) {
          myGLCD.setFont(BigFont);
          myGLCD.setColor(VGA_RED);
          myGLCD.print("ERORR",360, 250);
        } else {
          myGLCD.setFont(SevenSegNumFont);
          myGLCD.setColor(VGA_SILVER);
          myGLCD.printNumI(tc2,350, 232);
        }
      }
    //Настройка экрана Рабочий режим
      if (updateScreen) {
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("РАБОЧИЙ РЕЖИМ",137, 5);
        myGLCD.textRus("`",195, 123);
        myGLCD.textRus("`",195, 235);
        myGLCD.textRus("`",450, 123);
        myGLCD.textRus("`",450, 235);
        myGLCD.setColor(VGA_BLUE);
        myGLCD.textRus("ТЕКУЩИЙ ПРОФИЛЬ:",LEFT, 40);
        myGLCD.textRus("ВЕРХНИЙ НАГРЕВАТЕЛЬ",90, 80);
        myGLCD.textRus("НИЖНИЙ НАГРЕВАТЕЛЬ",101, 194);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.drawRoundRect(2,108,478,179);
        myGLCD.drawRoundRect(2,221,478,291);
        myGLCD.drawRect(224,108,478,179);
        myGLCD.drawRect(224,221,478,291);
        myGLCD.textRus("УСТАН",13, 153);
        myGLCD.textRus("УСТАН",13, 266);
        myGLCD.textRus("НАГРЕВ",235, 153);
        myGLCD.textRus("НАГРЕВ",235, 266);
        updateScreen = false;
      }
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(currentProfile,260, 40);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(temperatureStep[editStep],90, 120);
        myGLCD.printNumI(SP2,90, 232);
        windowStartTime = millis();
      
      if (upSwitchState == HIGH && ( millis() - ms_button)>500)//if up switch is pressed go to next profile
      {
        ms_button =  millis();
        currentProfile = currentProfile + 1;
        if (currentProfile >= 6)//if currentProfile = 5 and up is pressed go back to profile 1
        {
          currentProfile = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>500)//same as above just go down one profile
      {
        ms_button =  millis();
        currentProfile = currentProfile - 1;
        if (currentProfile <= 0)
        {
          currentProfile = 5;
        }
      }
      loadProfile();//call the loadProfile function to load from eeprom
      
//фиксируем момент нажатия кнопки "ОК" + защита от дребезга
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
        button_long_state = false;
      }
//держим "ОК" 3секунды и заходим в меню настроек      
      if (okSwitchState == HIGH && !button_long_state && ( millis() - ms_button)>3000)
      {
        button_long_state = true;
        button_state = false;
        reflowState = REFLOW_STATE_MENU_STEPS;
        //update next screen
        updateScreen = true;
      }
//включаем пайку, кнопка сработает после отпускания "ОК"       
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        tone(buzzerPin, 1045, 500);  //звуковой сигнал при старте профиля
        //update next screen
        updateScreen = true;
        reflowStatus = REFLOW_STATUS_ON;
        reflowState = REFLOW_STATE_STEP_RAMP;
      }
      break;
//устанавливаем количество шагов профиля
    case REFLOW_STATE_MENU_STEPS:
      if (updateScreen){
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_BLUE);
        myGLCD.textRus("НАСТРОЙКА           ПРОФИЛЬ",0, 5);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("К-ВО ШАГОВ",10, 35);
        myGLCD.textRus("РАЗМЕР НИЗА",260, 35);
        myGLCD.textRus("ТЕМПЕРАТУРА НИЗА",105, 90);
        myGLCD.textRus("МОЩНОСТЬ НИЗ",10, 205);
        myGLCD.textRus("МОЩНОСТЬ ВЕРХ",260, 205);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(5,28,200,56);
        myGLCD.printNumI(currentProfile,440, 5);
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(profileSteps,180, 35);
        updateScreen = false;
      }
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(profileSteps,180, 35);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(SP2,175, 132);
        myGLCD.printNumI(pwr_BOTTOM,95, 250);
        myGLCD.printNumI(pwr_TOP,350, 250);
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(tableSize,450, 35);
        //myGLCD.printNumI("%",100, 250);
        //myGLCD.printNumI("%",380, 250);

      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        profileSteps = profileSteps + 1;
        if (profileSteps >= 10)
        {
          profileSteps = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        profileSteps = profileSteps - 1;
        if (profileSteps <= 0) 
        {
          profileSteps = 9;
        }
      }
      
      if (okSwitchState == LOW && button_long_state) button_long_state = false; //чтобы после входа в меню не кнопка "ОК" не считалась нажатой
      
      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TABLE_SIZE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//устанавливаем размер стола
    case REFLOW_STATE_MENU_TABLE_SIZE:
      if (updateScreen){
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(5,28,200,56);
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("РАЗМЕР НИЗА",260, 35);
        myGLCD.printNumI(profileSteps,180, 35);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(SP2,175, 132);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(250,28,475,56);
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(tableSize,450, 35);
        updateScreen = false;
      }
        myGLCD.printNumI(tableSize,450, 35);

      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        tableSize = tableSize + 1;
        if (tableSize >= 4)
        {
          tableSize = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        tableSize = tableSize - 1;
        if (tableSize <= 0) 
        {
          tableSize = 3;
        }
      }    
      if (okSwitchState == LOW && button_long_state) button_long_state = false; //чтобы после входа в меню не кнопка "ОК" не считалась нажатой
      
      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button)>120)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//устанавливаем температуру "Нижнего Нагревателя"
    case REFLOW_STATE_MENU_BOTTOM_HEAT:
      if (updateScreen) {
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(profileSteps,180, 35);
        myGLCD.printNumI(tableSize,450, 35);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(5,28,200,56);
        myGLCD.drawRoundRect(250,28,475,56);
        updateScreen = false;
      }
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(95,85,370,113);
        myGLCD.printNumI(SP2,175, 132);

      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 + 1;
        if (Setpoint2 >= 250)
        {
          Setpoint2 = 250;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 - 1;
        if (Setpoint2 <= 100)
        {
          Setpoint2 = 100;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"     
     case REFLOW_STATE_MENU_BOTTOM_PWR:    
      if (updateScreen)
      {        
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(95,85,370,113);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(SP2,175, 132);
        myGLCD.setFont(BigFontRus);
        myGLCD.textRus("МОЩНОСТЬ НИЗ",10, 205);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(5,200,205,223);
        updateScreen = false;
      }
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(pwr_BOTTOM,95, 250);
      
       if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        pwr_BOTTOM = pwr_BOTTOM + 1;
        if (pwr_BOTTOM >= 100)
        {
          pwr_BOTTOM = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        pwr_BOTTOM = pwr_BOTTOM - 1;
        if (pwr_BOTTOM <= 0)
        {
         pwr_BOTTOM = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_PWR;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"      
     case REFLOW_STATE_MENU_TOP_PWR:      
      if (updateScreen)
      {        
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(5,200,205,223);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(pwr_BOTTOM,95, 250);
        myGLCD.setFont(BigFontRus);
        myGLCD.textRus("МОЩНОСТЬ ВЕРХ",260, 205);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(255,200,473,223);
        updateScreen = false;
      }
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(pwr_TOP,350, 250);
      
       if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        pwr_TOP = pwr_TOP + 1;
        if (pwr_TOP >= 100)
        {
          pwr_TOP = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        pwr_TOP = pwr_TOP - 1;
        if (pwr_TOP <= 0)
        {
         pwr_TOP = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_STEP_RAMP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//устанавливаем скорость нагрева "Верхним Нагревателем"
    case REFLOW_STATE_MENU_STEP_RAMP:
      if (updateScreen) {
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_BLUE);
        myGLCD.textRus("НАСТРОЙКА           ПРОФИЛЬ",0, 5);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("СКОРОСТЬ НАГРЕВА ВЕРХОМ",40, 35);
        myGLCD.textRus("ШАГ:",0, 80);
        myGLCD.textRus("ВРЕМЯ:",340, 80);
        myGLCD.textRus("ТЕМПЕРАТУРА ВЕРХА",100, 120);
        myGLCD.textRus("  ВРЕМЯ ПЕРЕХОДА НА СЛЕД ШАГ",0, 225);
        myGLCD.textRus("ШАГ:",0, 275);
        myGLCD.textRus("ВРЕМЯ:",300, 275);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(30,26,420,55);
        myGLCD.printNumI(currentProfile,440, 5);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[editStep],180, 160);
        myGLCD.printNumI(editStep + 1,70, 258);
        myGLCD.printNumI(dwellTimerStep[editStep],400, 258);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(editStep + 1,65, 60);
        updateScreen = false;
      }
        myGLCD.printNumI(rampRateStep[editStep] / 4,435, 60);       
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] + 1.;
        if (rampRateStep[editStep] >= 12.)  //(только целые числа) максимальная скорость роста температуры делённая на 4
        {
          rampRateStep[editStep] = 12.;  //(только целые числа) максимальная скорость роста температуры делённая на 4
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] - 1.;
        if (rampRateStep[editStep] <= 1.)  //(только целые числа) минимальная скорость роста температуры делённая на 4
        {
          rampRateStep[editStep] = 1.;  //(только целые числа) минимальная скорость роста температуры делённая на 4
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        //myGLCD.clrScr();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 //устанавливаем температуру "Верхнего Нагревателя"
    case REFLOW_STATE_MENU_STEP_TARGET:
      if (updateScreen) {
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(rampRateStep[editStep] / 4,435, 60);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(30,26,420,55); 
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(95,111,380,140);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(editStep + 1,65, 60);
        myGLCD.printNumI(temperatureStep[editStep],180, 160);       
        updateScreen = false;
      }
        myGLCD.printNumI(temperatureStep[editStep],180, 160);
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] + 1;
        if (temperatureStep[editStep] >= 250)
        {
          temperatureStep[editStep] = 250;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] - 1;
        if (temperatureStep[editStep] <= 0)
        {
          temperatureStep[editStep] = 0;
        }
        if (temperatureStep[editStep] <= 99)
        {
         myGLCD.printNumI(temperatureStep[editStep],180, 160);
        }
        if (temperatureStep[editStep] <= 9)
        {
         myGLCD.printNumI(temperatureStep[editStep],180, 160);
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_DWELL;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//устанавливаем время перехода на следующий шаг
    case REFLOW_STATE_MENU_STEP_DWELL:
      if (updateScreen) {
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(editStep + 1,65, 60);
        myGLCD.printNumI(temperatureStep[editStep],180, 160);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(95,111,380,140);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(25,218,452,245);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(editStep + 1,70, 258);
        myGLCD.printNumI(dwellTimerStep[editStep],400, 258);
        updateScreen = false;
      }
        myGLCD.printNumI(dwellTimerStep[editStep],400, 258);
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        dwellTimerStep[editStep] = dwellTimerStep[editStep] + 5;
        if (dwellTimerStep[editStep] >= 90)
        {
          dwellTimerStep[editStep] = 90;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        dwellTimerStep[editStep] = dwellTimerStep[editStep] - 5;
        if (dwellTimerStep[editStep] <= 0)
        {
          dwellTimerStep[editStep] = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_BOTTOM_P;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//настройка "ПИД" нижнего нагревателя
    case REFLOW_STATE_MENU_BOTTOM_P:
      if (updateScreen) {
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_BLUE);
        myGLCD.textRus("НАСТРОЙКА ПИД",130, 5);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("НИЖНИЙ НАГРЕВАТЕЛЬ",90, 35);
        myGLCD.textRus("ВЕРХНИЙ НАГРЕВАТЕЛЬ",85, 170);
        myGLCD.setFont(BigFont);
        myGLCD.print("P=",55, 70);
        myGLCD.print("I=",230, 70);
        myGLCD.print("D=",405, 70);
        myGLCD.print("P=",55, 205);
        myGLCD.print("I=",230, 205);
        myGLCD.print("D=",405, 205);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(80,28,390,55);
        myGLCD.printNumI(kp2,20, 100);
        updateScreen = false;
      }
        myGLCD.printNumI(kp2,20, 100);
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp2 = kp2 + 1;
        if (kp2 >= 500)
        {
          kp2 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp2 = kp2 - 1;
        if (kp2 <= 0)
        {
          kp2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_I:
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(kp2,20, 100);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(ki2/10,200, 100);       
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki2 = ki2 + 1;
        if (ki2 >= 500)
        {
          ki2 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki2 = ki2 - 1;
        if (ki2 <= 0)
        {
          ki2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_BOTTOM_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_D:
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(ki2/10,200, 100);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(kd2,370, 100);
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd2 = kd2 + 1;
        if (kd2 >= 500)
        {
          kd2 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd2 = kd2 - 1;
        if (kd2 <= 0)
        {
          kd2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_TOP_P;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//настройка "ПИД" верхнего нагревателя
    case REFLOW_STATE_MENU_TOP_P:
      if (updateScreen) {
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(kd2,370, 100);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(80,28,390,55);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(80,163,394,190);
        myGLCD.printNumI(kp1,20, 240);
        updateScreen = false;
      }
        myGLCD.printNumI(kp1,20, 240);
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp1 = kp1 + 1;
        if (kp1 >= 500)
        {
          kp1 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp1 = kp1 - 1;
        if (kp1 <= 0)
        {
          kp1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_I:
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(kp1,20, 240);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(ki1/10,200, 240);
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki1 = ki1 + 1;
        if (ki1 >= 500)
        {
          ki1 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki1 = ki1 - 1;
        if (ki1 <= 0)
        {
          ki1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_TOP_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_D:
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(ki1/10,200, 240);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(kd1,370, 240);
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd1 = kd1 + 1;
        if (kd1 >= 500)
        {
          kd1 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd1 = kd1 - 1;
        if (kd1 <= 0)
        {
          kd1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
//сохраняем текущие параметры в "Память"
        EEPROM.write((currentProfile - 1) * 29, profileSteps);
        EEPROM.write((currentProfile - 1) * 29 + 1, Setpoint2);
        for (int i = 0; i < 9; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 2), rampRateStep[i] );
          i++;
        }
        for (int i = 0; i < 9; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 11), dwellTimerStep[i] / 5);
          i++;
        }
        for (int i = 0; i < 9; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 20), temperatureStep[i]);
          i++;
        }
        EEPROM.write(((currentProfile - 1) * 6 + 122), kp1);
        EEPROM.write(((currentProfile - 1) * 6 + 123), ki1);
        EEPROM.write(((currentProfile - 1) * 6 + 124), kd1);
        EEPROM.write(((currentProfile - 1) * 6 + 125), kp2);
        EEPROM.write(((currentProfile - 1) * 6 + 126), ki2);
        EEPROM.write(((currentProfile - 1) * 6 + 127), kd2);

        EEPROM.write(((currentProfile - 1) * 6 + 128), tableSize);
        EEPROM.write(((currentProfile - 1) * 6 + 129), pwr_TOP);
        EEPROM.write(((currentProfile - 1) * 6 + 130), pwr_BOTTOM);
        
        myGLCD.clrScr();
        reflowState = REFLOW_STATE_IDLE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        myGLCD.clrScr();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//"Старт и процесс пайки", рост температуры с заданной скоростью
    case REFLOW_STATE_STEP_RAMP:
      if (updateScreen) {
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("СТАРТ",360, 40);
        myGLCD.setColor(VGA_WHITE);
        myGLCD.textRus("РАБОЧИЙ РЕЖИМ",140, 5);
        myGLCD.textRus("ШАГ",5, 300);
        myGLCD.textRus("ВРЕМЯ",100, 300);
        myGLCD.textRus("СКОРОСТЬ",280, 300);
        myGLCD.setColor(250, 180, 000);
        myGLCD.textRus("УСТАН",13, 153);
        myGLCD.textRus("УСТАН",13, 266);
        myGLCD.textRus("НАГРЕВ",235, 153);
        myGLCD.textRus("НАГРЕВ",235, 266);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(VGA_RED);
        myGLCD.printNumI(temperatureStep[editStep],90, 120);
        myGLCD.printNumI(SP2,90, 232);
        updateScreen = false;
      }
//фиксируем размер стола
    case REFLOW_STATE_TABLE_SIZE:
      if (tableSize == 1) {
       digitalWrite(P1_PIN, LOW);
       }
      if (tableSize == 2) {
       digitalWrite(P1_PIN, LOW);
       digitalWrite(P2_PIN, LOW);
       }
      if (tableSize == 3) {
       digitalWrite(P1_PIN, LOW);
       digitalWrite(P2_PIN, LOW);
       digitalWrite(P3_PIN, LOW);
       }
      if (tc1 == 115) tone(buzzerPin, 1045, 500);  //звуковой сигнал              
      if (tc1 >= 115) digitalWrite(P4_PIN, LOW);
      if (tc1 >= 118 && !TopStart) TopStart = true;  //если температура низа вышла на уставку

      if (TopStart == true)   // включаем верхний нагреватель

      { //открываем cкобку (чтоб не запутаться)
        
//фиксируем стартовую температуру      
      if (flag == 0)
      {
        startTemp = tc1;
        flag = 1;
      }
//устанавливаем нужный шаг, до которой нагрета плата      
      if (startTemp > temperatureStep[currentStep -1]){
        for (x=1; startTemp > temperatureStep[currentStep - 1];currentStep++){
          x++;
      }
       }      
      if (currentStep > x && flag == 1){
        flag = 0;
        startTemp = temperatureStep[currentStep - 2];        
        flag = 1;
      }
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(currentStep,60, 300);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(VGA_RED);
        myGLCD.printNumI(SP2,90, 232);
      
//счётчик скорости роста температуры                  
      if ((currentMillis - previousMillis) > 4000 / rampRateStep[currentStep - 1]) //seconds counter
      {
        previousMillis = currentMillis;
        counter = counter + 1;
        setpointRamp = counter + startTemp;
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(setpointRamp,420, 300);
        Setpoint1 = setpointRamp;
      }
       } //закрывам скобку
 
      if (setpointRamp >= temperatureStep[currentStep - 1]) //если достигли нужной температуры
      {
        myGLCD.setColor(VGA_RED);
        myGLCD.printNumI(temperatureStep[currentStep - 1],420, 300);
        reflowState = REFLOW_STATE_STEP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>50)
      {
        ms_button =  millis();
        digitalWrite(P1_PIN, HIGH);
        digitalWrite(P2_PIN, HIGH);
        digitalWrite(P3_PIN, HIGH);
        digitalWrite(P4_PIN, HIGH);
        
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
      }
      break;
      
    case REFLOW_STATE_STEP:
      Setpoint1 = temperatureStep[currentStep - 1];
      if (Input1 >= temperatureStep[currentStep - 1])
      {
        counter = 0;
        reflowState = REFLOW_STATE_STEP_DWELL;        
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        digitalWrite(P1_PIN, HIGH);
        digitalWrite(P2_PIN, HIGH);
        digitalWrite(P3_PIN, HIGH);
        digitalWrite(P4_PIN, HIGH);
        updateScreen = true;
        
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//считаем время перехода на следующий шаг
    case REFLOW_STATE_STEP_DWELL:
      if (currentMillis - previousMillis > 1000) 
      {
        previousMillis = currentMillis;
        counter = counter + 1;
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(dwellTimerStep[currentStep - 1],190, 300);
      }
      if (counter == dwellTimerStep[currentStep - 1]) //если счётчик равен установленному времени
      {
        myGLCD.setFont(BigFont);
        myGLCD.setColor(VGA_RED);
        myGLCD.printNumI(dwellTimerStep[currentStep - 1],190, 300);
        tone(buzzerPin, 1045, 500);  //звуковой сигнал
        counter = 0;
        setpointRamp = 0;
        if (profileSteps == currentStep) //если достигли последнего шага 
        {
          currentStep = 1;
          x = 1;      //устанавливаем переменную в исходное состояние
          flag = 0;   //после завершения профиля сбрасываем флаг
          reflowState = REFLOW_STATE_COMPLETE;
        }
        else //если шаг не последний
        {          
          currentStep++; //переходим на следующий шаг
          reflowState = REFLOW_STATE_STEP_RAMP;          
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        digitalWrite(P1_PIN, HIGH);
        digitalWrite(P2_PIN, HIGH);
        digitalWrite(P3_PIN, HIGH);
        digitalWrite(P4_PIN, HIGH);
        updateScreen = true;
        
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//завершение пайки      
    case REFLOW_STATE_COMPLETE:
       digitalWrite(P1_PIN, HIGH);
       digitalWrite(P2_PIN, HIGH);
       digitalWrite(P3_PIN, HIGH);
       digitalWrite(P4_PIN, HIGH);
                  
      reflowStatus = REFLOW_STATUS_OFF;
      reflowState = REFLOW_STATE_IDLE;      
      updateScreen = true;
      break;
      }
//включение нагревателей
    if (reflowStatus == REFLOW_STATUS_ON)
    {
    if (millis() > nextRead1)
    {
      Input1 = filter1(thermocouple1.readCelsius());
      Input2 = filter2(thermocouple2.readCelsius());
      tc1 = Input1;
      tc2 = Input2;
    nextRead1 += SENSOR_SAMPLING_TIME;
    if (isnan(Input1)) {
       myGLCD.setFont(BigFont);
       myGLCD.setColor(VGA_RED);
       myGLCD.print("ERORR",360, 140);
    } else {
       myGLCD.setFont(SevenSegNumFont);
       myGLCD.setColor(250, 180, 000);
       myGLCD.printNumI(tc1,350, 120);
    }
    if (isnan(Input2)) {
       myGLCD.setFont(BigFont);
       myGLCD.setColor(VGA_RED);
       myGLCD.print("ERORR",360, 250);
    } else {
       myGLCD.setFont(SevenSegNumFont);
       myGLCD.setColor(250, 180, 000);
       myGLCD.printNumI(tc2,350, 232);
    }
     }
     
      myPID1.SetTunings(kp1, ki1/10, kd1);
      myPID2.SetTunings(kp2, ki2/10, kd2);
      myPID1.Compute();
      myPID2.Compute();
      attachInterrupt(0, Dimming, RISING); // настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
      /* -------------------------------------------------------------------------------
      LOW вызывает прерывание, когда на порту LOW
      CHANGE прерывание вызывается при смене значения на порту, с LOW на HIGH и наоборот
      RISING прерывание вызывается только при смене значения на порту с LOW на HIGH
      FALLING прерывание вызывается только при смене значения на порту с HIGH на LOW   
     ------------------------------------------------------------------------------- */
      if (tc1 >= 240)
      {
       digitalWrite(P1_PIN, HIGH);
       digitalWrite(P2_PIN, HIGH);
       digitalWrite(P3_PIN, HIGH);
       digitalWrite(P4_PIN, HIGH);
       tone(buzzerPin, 1045, 400);  //звуковой сигнал
       
       reflowStatus = REFLOW_STATUS_OFF;
       reflowState = REFLOW_STATE_IDLE;    
       updateScreen = true;
       }
         
      if (!TopStart) Output1=0;        
       }
     else
       {
        digitalWrite(RelayPin1, LOW);
        digitalWrite(RelayPin2, LOW);
       }
       }

void Dimming()
{
  OutPWR_TOP();
  OutPWR_BOTTOM();
}

void OutPWR_TOP()
{   
  reg1 = round(Output1*(pwr_TOP*0.01)) + er1; //pwr- задание выходной мощности в %, er- ошибка округления
  
     if (reg1 < 50)
       {
        out1=LOW;
        
         er1 = reg1 ; // reg- переменная для расчетов
       }
          else 
        {
          out1=HIGH;
          
          er1=reg1-100;
        } 
   digitalWrite(RelayPin1,out1);//пин через который осуществляется дискретное управление
   
}

void OutPWR_BOTTOM()
{
  reg2 = round(Output2*(pwr_BOTTOM*0.01)) + er2; //pwr- задание выходной мощности в %, er- ошибка округления

     if (reg2 < 50)
       {
        out2=LOW;
        
         er2 = reg2 ; // reg- переменная для расчетов
       }
          else 
        {
          out2=HIGH;
          
          er2=reg2-100;
        }    
   digitalWrite(RelayPin2,out2);//пин через который осуществляется дискретное управление
   //i++;  
 }
         
  float filter1(float val_1) {  //функция фильтрации показений "Верхней" термопары
  Pc1 = P1 + varProcess1;
  G1 = Pc1/(Pc1 + varTerm1);
  P1 = (1-G1)*Pc1;
  Xp1 = Xe1;
  Zp1 = Xp1;
  Xe1 = G1*(val_1-Zp1)+Xp1; // "фильтрованное" значение
  return(Xe1);
  }
  float filter2(float val_2) {  //функция фильтрации показений "Нижней" термопары
  Pc2 = P2 + varProcess2;
  G2 = Pc2/(Pc2 + varTerm2);
  P2 = (1-G2)*Pc2;
  Xp2 = Xe2;
  Zp2 = Xp2;
  Xe2 = G2*(val_2-Zp2)+Xp2; // "фильтрованное" значение
  return(Xe2);
  }
